%%
%%% Fitting the incubation curve F_th(N) with the generic model
%%% published in: Z. Sun, M. Lenzner, and W. Rudolph, Journ. Appl.
%%% Phys. vol.115 (2015) ......
%%%
%%% INPUT:
%%%
%%% Data are read from file FvsN.csv, a comma-delimited list, where each
%%% line is one pair of corresponding values N and F_th(N) in J/cm.
%%% A single-shot threshold (i.e. a line with N=1) has to be included,
%%% since it is needed as a parameter for the fit procedure. If no such
%%% experimental value is present, you must guess one, replace the xx in
%%% line 54 with this value and uncomment the line.
%%% 
%%% OUTPUT:
%%%
%%% A graph that shows experimental values, fitted curve and the 
%%% calculated coefficients
%%% 
%%% A Microsoft Excel file FvsN_result.xls with 3 sheets containing
%%%        (1) the experimental values from FvsN.csv
%%%        (2) some points of the fitted curve within the range
%%%            given by the experimental values
%%%        (3) the coefficients of the fitted curve
%%% Attention: the file will be relentlessly overwritten by each
%%% new run of the script
 

clc
clear

%% Reading data from file
datafilename=['FvsN'];
R1 = csvread([datafilename,'.csv']) ; R2 = R1';
ExN = R2(1,:) ; 
Fth = R2(2,:) ;
Nmax = numel(ExN);

%% sorting by ascending pulse number
FF = [ ExN ; Fth ] ;      % build matrix FF for sorting
[Y,I]=sort(FF(1,:)) ;     % get sort index ascending ExN 
FF=FF(:,I) ;              % sort FF according to index
ExN = FF(1,:) ;           % extract sorted vector of pulse numbers
Fth = FF(2,:) ;           % extract sorted vector of threshold fluences

%% Fit using LSQNONLIN, F1 and Finf = first and last value 
%% of experimental data

options = optimoptions(@lsqnonlin,'TolX',1e-12,'TolFun',1e-12, ...
     'MaxFunEvals',1e8,'MaxIter',1e8)
lb=double.empty(1,1,0);ub=lb;

F1 = Fth(1) ;
% F1 = xx ;
Finf = Fth(Nmax) ;
x0 = [2 1 .02];
FitFun = @(x) WRpmBoth(F1,Finf,x(1),x(2),x(3),Fth,ExN); 
fitres=lsqnonlin(FitFun,x0,lb,ub,options)
Dalpha=fitres(1);beta=fitres(2);gamma=fitres(3);

%% construct a vector with a reasonable size that contains enough data
%  to plot a smooth curve of the fit function in a semilogx plot
%  i.e. N = 1,2,3, ....... 10,20,30, ........ 100,200,300, .......
nn = 1; i6 = 1;             % nn - actual number of pulses, i6 - counter
while nn < max(ExN);
    Fci(i6) = nn;
    FCurve(i6) = WRBoth(F1,Finf,Dalpha,beta,gamma,nn);
    i6 = i6 + 1;
    nn = nn + 10^floor(log10(nn+1));
end

%% calculate RMS of fit
RMS=0;
for i3 = (1:Nmax)
    abw(i3) = Fth(i3) - WRBoth(F1,Finf,Dalpha,beta,gamma,ExN(i3));
RMS = RMS + (abw(i3))^2;
end
RMS = sqrt(RMS/Nmax);
%% plot

figure;
semilogx(ExN,Fth,'--rs','LineStyle','none',...   % plot experimental values
    'Marker','D',...
    'MarkerEdgeColor','k',...
    'MarkerFaceColor','r',...
    'MarkerSize',10);
ylabel('threshold fluence [ J/cm ]');
xlabel('Number of Pulses');
hold all;
semilogx(Fci,FCurve);
sttr = {['\Delta \alpha / \alpha_0 =  ',num2str(Dalpha,'%2.3f')], ...
        ['\beta = ',num2str(beta,'%1.5f')], ...
        ['\gamma = ',num2str(gamma,'%1.5f')] } ;
annotation('textbox',[.6 .6 .1 .1],'String',sttr) ;

EXP = cat(2,ExN.',Fth.') ;
THE = cat(2,Fci.',FCurve.') ;
Csraw = ['F_1  ';'F_inf';'Da/a ';'beta ';'gamma'] 
Cstr = cellstr(Csraw) ;
Cval = [ F1 Finf Dalpha beta gamma ] ; Cval = Cval' ;
datafilename = [datafilename,'_result'] ;
xlswrite(datafilename,EXP,'Experiment') ; 
xlswrite(datafilename,THE,'Model') ; 
xlswrite(datafilename,Cstr,'Fit parameters','A1') ;
xlswrite(datafilename,Cval,'Fit parameters','B1') ;